@@ -13,8 +13,8 @@
 
 TODO:
 * Where is Transf_gen for general monotonic transformation ? found and added it
-* write some docstrings, some parts I don't remember
-* add Box-Cox transformation, parameterized ?
+* write some docstrings, some parts I do not remember
+* add Box-Cox transformation, parametrized ?
 
 
 this is only partially cleaned, still includes test examples as functions
@@ -49,7 +49,6 @@ Author: josef-pktd
 License: BSD
 
 '''
-from __future__ import print_function
 
 import numpy as np
 from numpy import poly1d,sqrt, exp
@@ -58,7 +57,6 @@ import scipy
 from scipy import stats, special
 from scipy.stats import distributions
 
-from statsmodels.compat.python import range, iteritems
 from statsmodels.stats.moment_helpers import mvsk2mc, mc2mvsk
 
 
@@ -232,7 +230,6 @@ def pdf_moments_st(cnt):
 
     version of scipy.stats, any changes ?
     the scipy.stats version has a bug and returns normal distribution
-
     """
 
     N = len(cnt)
@@ -302,7 +299,7 @@ def pdf_mvsk(mvsk):
 
     References
     ----------
-    http://en.wikipedia.org/wiki/Edgeworth_series
+    https://en.wikipedia.org/wiki/Edgeworth_series
     Johnson N.L., S. Kotz, N. Balakrishnan: Continuous Univariate
     Distributions, Volume 1, 2nd ed., p.30
     """
@@ -350,7 +347,7 @@ def pdf_moments(cnt):
 
     References
     ----------
-    http://en.wikipedia.org/wiki/Edgeworth_series
+    https://en.wikipedia.org/wiki/Edgeworth_series
     Johnson N.L., S. Kotz, N. Balakrishnan: Continuous Univariate
     Distributions, Volume 1, 2nd ed., p.30
     """
@@ -415,7 +412,7 @@ class NormExpan_gen(distributions.rv_continuous):
         where xc = (x-mu)/sig is the standardized value of the random variable
         and H(xc,3) and H(xc,4) are Hermite polynomials
 
-        Note: This distribution has to be parameterized during
+        Note: This distribution has to be parametrized during
         initialization and instantiation, and does not have a shape
         parameter after instantiation (similar to frozen distribution
         except for location and scale.) Location and scale can be used
@@ -495,7 +492,7 @@ License: BSD
 
 def get_u_argskwargs(**kwargs):
     #Todo: What's this? wrong spacing, used in Transf_gen TransfTwo_gen
-    u_kwargs = dict((k.replace('u_','',1),v) for k,v in iteritems(kwargs)
+    u_kwargs = dict((k.replace('u_','',1),v) for k,v in kwargs.items()
                     if k.startswith('u_'))
     u_args = u_kwargs.pop('u_args',None)
     return u_args, u_kwargs
@@ -735,14 +732,11 @@ class TransfTwo_gen(distributions.rv_continuous):
                                 longname = longname, extradoc = extradoc)
 
         # add enough info for self.freeze() to be able to reconstruct the instance
-        try:
-            self._ctor_param.update(dict(kls=kls, func=func,
-                    funcinvplus=funcinvplus, funcinvminus=funcinvminus,
-                    derivplus=derivplus, derivminus=derivminus,
-                    shape=self.shape))
-        except AttributeError:
-            # scipy < 0.14 does not have this, ignore and do nothing
-            pass
+        self._ctor_param.update(
+            dict(kls=kls, func=func, funcinvplus=funcinvplus,
+                 funcinvminus=funcinvminus, derivplus=derivplus,
+                 derivminus=derivminus, shape=self.shape)
+        )
 
     def _rvs(self, *args):
         self.kls._size = self._size   #size attached to self, not function argument
@@ -795,7 +789,7 @@ class SquareFunc(object):
     '''class to hold quadratic function with inverse function and derivative
 
     using instance methods instead of class methods, if we want extension
-    to parameterized function
+    to parametrized function
     '''
     def inverseplus(self, x):
         return np.sqrt(x)
@@ -906,7 +900,7 @@ a wrapper for scipy.stats.kde.mvndst
 *     CORREL REAL, array of correlation coefficients; the correlation
 *            coefficient in row I column J of the correlation matrix
 *            should be stored in CORREL( J + ((I-2)*(I-1))/2 ), for J < I.
-*            THe correlation matrix must be positive semidefinite.
+*            The correlation matrix must be positive semidefinite.
 *     MAXPTS INTEGER, maximum number of function values allowed. This
 *            parameter can be used to limit the time. A sensible
 *            strategy is to start with MAXPTS = 1000*N, and then
@@ -1026,7 +1020,7 @@ def mvstdnormcdf(lower, upper, corrcoef, **kwds):
 
     '''
     n = len(lower)
-    #don't know if converting to array is necessary,
+    #do not know if converting to array is necessary,
     #but it makes ndim check possible
     lower = np.array(lower)
     upper = np.array(upper)
